@@ -1,5 +1,5 @@
 import math
-from collections.abc import Callable, Mapping, MutableMapping, MutableSequence, Sequence, Set
+from collections.abc import Callable, Generator, Mapping, MutableMapping, MutableSequence, Sequence, Set
 from typing import Any, SupportsFloat, SupportsInt
 
 from iker.common.utils.numutils import is_normal_real
@@ -24,6 +24,8 @@ __all__ = [
     "json_traverse",
     "json_reformat",
     "json_sanitize",
+    "json_difference",
+    "json_equals",
     "json_compare",
 ]
 
@@ -347,7 +349,107 @@ def json_sanitize(obj: Any, *, str_inf_nan: bool = True, str_unregistered: bool
                          unregistered_formatter=unregistered_formatter)
 
 
-def json_compare(
+def json_difference(
+    a: JsonTypeCompatible,
+    b: JsonTypeCompatible,
+    node_path: NodePath | None = None,
+    *,
+    int_strict: bool = False,
+    float_tol: float = 1e-5,
+    list_order: bool = True,
+    dict_extra: bool = False,
+) -> Generator[tuple[NodePath, str], None, None]:
+    """
+    Compares two JSON-like structures and yields differences found, with options for integer strictness, float
+    tolerance, list order, and dictionary key matching.
+
+    :param a: The first JSON-compatible object to compare.
+    :param b: The second JSON-compatible object to compare.
+    :param node_path: The current node path during recursion (used internally).
+    :param int_strict: Whether to require strict integer type matching.
+    :param float_tol: The tolerance for comparing float values.
+    :param list_order: Whether to require list order to match.
+    :param dict_extra: Whether to allow extra keys in dictionaries.
+    :return: Tuples of node paths and difference descriptions.
+    """
+    if a is None or b is None:
+        if not (a is None and b is None):
+            yield node_path, "one value is None while the other is not"
+        return
+
+    if isinstance(a, (str, bool)):
+        if type(a) != type(b):
+            yield node_path, f"type mismatch: '{type(a)}' vs '{type(b)}'"
+        elif a != b:
+            yield node_path, f"value mismatch: '{a}' vs '{b}'"
+        return
+
+    if isinstance(a, (SupportsFloat, SupportsInt)) and isinstance(b, (SupportsFloat, SupportsInt)):
+        isint_a = isinstance(a, int) or not isinstance(a, SupportsFloat)
+        isint_b = isinstance(b, int) or not isinstance(b, SupportsFloat)
+        if isint_a and isint_b:
+            if int(a) != int(b):
+                yield node_path, f"integer value mismatch: '{int(a)}' vs '{int(b)}'"
+            return
+        if int_strict and (isint_a or isint_b):
+            yield node_path, "integer type mismatch under strict mode"
+            return
+        va = int(a) if isint_a else float(a)
+        vb = int(b) if isint_b else float(b)
+        if math.isnan(va) or math.isnan(vb):
+            if not (math.isnan(va) and math.isnan(vb)):
+                yield node_path, "NaN mismatch"
+            return
+        if math.isinf(va) and math.isinf(vb):
+            if va != vb:
+                yield node_path, "infinity sign mismatch"
+            return
+        if abs(va - vb) > float_tol:
+            yield node_path, f"float value mismatch: '{va}' vs '{vb}' with tolerance '{float_tol}'"
+        return
+
+    if isinstance(a, Mapping) and isinstance(b, Mapping):
+        if not dict_extra and set(a.keys()) != set(b.keys()):
+            yield node_path, f"dictionary key mismatch: '{set(a.keys())}' vs '{set(b.keys())}'"
+            return
+        for k in set(a.keys()) & set(b.keys()):
+            yield from json_difference(a[k],
+                                       b[k],
+                                       (node_path or []) + [k],
+                                       int_strict=int_strict,
+                                       float_tol=float_tol,
+                                       list_order=list_order,
+                                       dict_extra=dict_extra)
+        return
+
+    if isinstance(a, Sequence) and isinstance(b, Sequence):
+        if len(a) != len(b):
+            yield node_path, f"list length mismatch: '{len(a)}' vs '{len(b)}'"
+            return
+        if list_order:
+            for i, (va, vb) in enumerate(zip(a, b)):
+                yield from json_difference(va,
+                                           vb,
+                                           (node_path or []) + [i],
+                                           int_strict=int_strict,
+                                           float_tol=float_tol,
+                                           list_order=list_order,
+                                           dict_extra=dict_extra)
+        else:
+            for i, (va, vb) in enumerate(zip(sorted(a), sorted(b))):
+                yield from json_difference(va,
+                                           vb,
+                                           (node_path or []) + [i],
+                                           int_strict=int_strict,
+                                           float_tol=float_tol,
+                                           list_order=list_order,
+                                           dict_extra=dict_extra)
+        return
+
+    yield node_path, f"type mismatch: '{type(a)}' vs '{type(b)}'"
+
+
+def json_equals(
     a: JsonTypeCompatible,
     b: JsonTypeCompatible,
     *,
@@ -357,8 +459,7 @@ def json_compare(
     dict_extra: bool = False,
 ) -> bool:
     """
-    Compares two JSON-like structures for equality, with options for integer strictness, float tolerance, list order,
-    and dictionary key matching.
+    Compares two JSON-like structures for equality based on specified criteria.
 
     :param a: The first JSON-compatible object to compare.
     :param b: The second JSON-compatible object to compare.
@@ -368,58 +469,14 @@ def json_compare(
     :param dict_extra: Whether to allow extra keys in dictionaries.
     :return: ``True`` if the structures are considered equal, ``False`` otherwise.
     """
-    if a is None or b is None:
-        return a is None and b is None
-
-    if isinstance(a, (str, bool)):
-        if type(a) != type(b):
-            return False
-        return a == b
-
-    if isinstance(a, (SupportsFloat, SupportsInt)) and isinstance(b, (SupportsFloat, SupportsInt)):
-        isint_a = isinstance(a, int) or not isinstance(a, SupportsFloat)
-        isint_b = isinstance(b, int) or not isinstance(b, SupportsFloat)
-        if isint_a and isint_b:
-            return int(a) == int(b)
-        if int_strict and (isint_a or isint_b):
-            return False
-        va = int(a) if isint_a else float(a)
-        vb = int(b) if isint_b else float(b)
-        if math.isnan(va) and math.isnan(vb):
-            return True
-        if math.isinf(va) and math.isinf(vb):
-            return va == vb
-        return abs(va - vb) <= float_tol
-
-    if isinstance(a, Mapping) and isinstance(b, Mapping):
-        if not dict_extra and set(a.keys()) != set(b.keys()):
-            return False
-        return all(json_compare(a[k],
-                                b[k],
+    return next(json_difference(a,
+                                b,
+                                node_path=[],
                                 int_strict=int_strict,
                                 float_tol=float_tol,
                                 list_order=list_order,
-                                dict_extra=dict_extra)
-                   for k in set(a.keys()) & set(b.keys()))
+                                dict_extra=dict_extra),
+                None) is None
 
-    if isinstance(a, Sequence) and isinstance(b, Sequence):
-        if len(a) != len(b):
-            return False
-        if list_order:
-            return all(json_compare(va,
-                                    vb,
-                                    int_strict=int_strict,
-                                    float_tol=float_tol,
-                                    list_order=list_order,
-                                    dict_extra=dict_extra)
-                       for va, vb in zip(a, b))
-        else:
-            return all(json_compare(va,
-                                    vb,
-                                    int_strict=int_strict,
-                                    float_tol=float_tol,
-                                    list_order=list_order,
-                                    dict_extra=dict_extra)
-                       for va, vb in zip(sorted(a), sorted(b)))
 
-    raise ValueError(f"incompatible type '{type(a)}' and '{type(b)}'")
+json_compare = json_equals
