@@ -1,7 +1,6 @@
 import itertools
 from collections import deque
 from collections.abc import Callable, Generator, Iterable, Iterator, Sequence, Sized
-from typing import Generic, TypeAlias, TypeVar
 from typing import overload
 
 __all__ = [
@@ -32,19 +31,13 @@ __all__ = [
     "seq",
 ]
 
-T = TypeVar("T")
-KT = TypeVar("KT")
-VT = TypeVar("VT")
-TT = TypeVar("TT")
-Tco = TypeVar("Tco", covariant=True)
-
-NestedIterable: TypeAlias = "T | Iterable[NestedSequence[T]]"
+type NestedIterable[T] = T | Iterable[NestedIterable[T]]
 
 
 # See Haskell's list operations head, tail, init, and last
 # which is also provided in Scala list operations
 
-def head(ms: Sequence[T]) -> T:
+def head[T](ms: Sequence[T]) -> T:
     """
     Returns the first element of the sequence ``ms``.
 
@@ -54,7 +47,7 @@ def head(ms: Sequence[T]) -> T:
     return ms[0]
 
 
-def head_or_none(ms: Sequence[T]) -> T | None:
+def head_or_none[T](ms: Sequence[T]) -> T | None:
     """
     Returns the first element of the sequence ``ms``, or ``None`` if the sequence is empty.
 
@@ -66,7 +59,7 @@ def head_or_none(ms: Sequence[T]) -> T | None:
     return None
 
 
-def last(ms: Sequence[T]) -> T:
+def last[T](ms: Sequence[T]) -> T:
     """
     Returns the last element of the sequence ``ms``.
 
@@ -76,7 +69,7 @@ def last(ms: Sequence[T]) -> T:
     return ms[-1]
 
 
-def last_or_none(ms: Sequence[T]) -> T | None:
+def last_or_none[T](ms: Sequence[T]) -> T | None:
     """
     Returns the last element of the sequence ``ms``, or ``None`` if the sequence is empty.
 
@@ -88,7 +81,7 @@ def last_or_none(ms: Sequence[T]) -> T | None:
     return None
 
 
-def tail(ms: Sequence[T]) -> Sequence[T]:
+def tail[T](ms: Sequence[T]) -> Sequence[T]:
     """
     Returns all elements of the sequence ``ms`` except the first.
 
@@ -98,7 +91,7 @@ def tail(ms: Sequence[T]) -> Sequence[T]:
     return ms[1:]
 
 
-def tail_iter(ms: Iterable[T]) -> Generator[T, None, None]:
+def tail_iter[T](ms: Iterable[T]) -> Generator[T, None, None]:
     """
     Returns an iterator over all elements of the iterable ``ms`` except the first.
 
@@ -114,7 +107,7 @@ def tail_iter(ms: Iterable[T]) -> Generator[T, None, None]:
         return
 
 
-def init(ms: Sequence[T]) -> Sequence[T]:
+def init[T](ms: Sequence[T]) -> Sequence[T]:
     """
     Returns all elements of the sequence ``ms`` except the last.
 
@@ -124,7 +117,7 @@ def init(ms: Sequence[T]) -> Sequence[T]:
     return ms[:-1]
 
 
-def init_iter(ms: Iterable[T]) -> Generator[T, None, None]:
+def init_iter[T](ms: Iterable[T]) -> Generator[T, None, None]:
     """
     Returns an iterator over all elements of the iterable ``ms`` except the last.
 
@@ -141,7 +134,7 @@ def init_iter(ms: Iterable[T]) -> Generator[T, None, None]:
         prev = this
 
 
-def some(x: T, test: Callable[[T], bool] = lambda x: x is not None) -> Generator[T, None, None]:
+def some[T](x: T, test: Callable[[T], bool] = lambda x: x is not None) -> Generator[T, None, None]:
     """
     Yields the value ``x`` if it passes the ``test`` function.
 
@@ -153,7 +146,7 @@ def some(x: T, test: Callable[[T], bool] = lambda x: x is not None) -> Generator
         yield x
 
 
-def flatten(ms: "NestedIterable[T] | None") -> Generator[T, None, None]:
+def flatten[T](ms: "NestedIterable[T] | None") -> Generator[T, None, None]:
     """
     Flattens a nested iterable ``ms`` into a generator of elements.
 
@@ -169,7 +162,7 @@ def flatten(ms: "NestedIterable[T] | None") -> Generator[T, None, None]:
             yield from flatten(m)
 
 
-def scan_left(ms: Sequence[T], zero: TT | None, func: Callable[[TT, T], TT]) -> Generator[TT, None, None]:
+def scan_left[T, R](ms: Sequence[T], zero: R | None, func: Callable[[R, T], R]) -> Generator[R, None, None]:
     """
     Applies a function cumulatively to the items of a sequence from left to right, yielding the intermediate results.
 
@@ -184,7 +177,7 @@ def scan_left(ms: Sequence[T], zero: TT | None, func: Callable[[TT, T], TT]) ->
         yield accum
 
 
-def scan_right(ms: Sequence[T], zero: TT | None, func: Callable[[TT, T], TT]) -> Generator[TT, None, None]:
+def scan_right[T, R](ms: Sequence[T], zero: R | None, func: Callable[[R, T], R]) -> Generator[R, None, None]:
     """
     Applies a function cumulatively to the items of a sequence from right to left, yielding the intermediate results.
 
@@ -199,7 +192,7 @@ def scan_right(ms: Sequence[T], zero: TT | None, func: Callable[[TT, T], TT]) ->
         yield accum
 
 
-def fold_left(ms: Sequence[T], zero: TT | None, func: Callable[[TT, T], TT]) -> TT | None:
+def fold_left[T, R](ms: Sequence[T], zero: R | None, func: Callable[[R, T], R]) -> R | None:
     """
     Applies a function cumulatively to the items of a sequence ``ms`` from left to right, reducing the sequence to a
     single value. If the initial value (``zero``) is ``None``, the first element of ``ms`` is used as the starting
@@ -219,7 +212,7 @@ def fold_left(ms: Sequence[T], zero: TT | None, func: Callable[[TT, T], TT]) ->
     return accum
 
 
-def fold_right(ms: Sequence[T], zero: TT | None, func: Callable[[TT, T], TT]) -> TT | None:
+def fold_right[T, R](ms: Sequence[T], zero: R | None, func: Callable[[R, T], R]) -> R | None:
     """
     Applies a function cumulatively to the items of a sequence ``ms`` from right to left, reducing the sequence to a
     single value. If the initial value (``zero``) is ``None``, the last element of ``ms`` is used as the starting
@@ -239,14 +232,14 @@ def fold_right(ms: Sequence[T], zero: TT | None, func: Callable[[TT, T], TT]) ->
     return accum
 
 
-def slide_left(
+def slide_left[T, R](
     ms: Sequence[T],
     window: int,
-    func: Callable[[Iterable[T]], TT] = lambda x: x,
+    func: Callable[[Iterable[T]], R] = lambda x: x,
     *,
     allow_partial: tuple[bool, bool] = (False, False),
     padding: tuple[T, T] = (None, None),
-) -> Generator[TT, None, None]:
+) -> Generator[R, None, None]:
     """
     Generates a sliding window over the input sequence from left to right.
 
@@ -295,14 +288,14 @@ def slide_left(
         yield from (func(ms[-i:]) for i in reversed(range(1, min(window, len(ms)))))
 
 
-def slide_right(
+def slide_right[T, R](
     ms: Sequence[T],
     window: int,
-    func: Callable[[Iterable[T]], TT] = lambda x: x,
+    func: Callable[[Iterable[T]], R] = lambda x: x,
     *,
     allow_partial: tuple[bool, bool] = (False, False),
     padding: tuple[T, T] = (None, None),
-) -> Generator[TT, None, None]:
+) -> Generator[R, None, None]:
     """
     Generates a sliding window over the input sequence from right to left.
 
@@ -352,28 +345,28 @@ def slide_right(
 
 
 @overload
-def grouped(
+def grouped[T, K](
     ms: Sequence[T],
-    key_func: Callable[[T], KT],
+    key_func: Callable[[T], K],
     keys_ordered: bool,
     values_only: False = False,
-) -> Generator[tuple[KT, list[T]], None, None]:
+) -> Generator[tuple[K, list[T]], None, None]:
     ...
 
 
 @overload
-def grouped(
+def grouped[T, K](
     ms: Sequence[T],
-    key_func: Callable[[T], KT],
+    key_func: Callable[[T], K],
     values_only: False = False,
-) -> Generator[tuple[KT, list[T]], None, None]:
+) -> Generator[tuple[K, list[T]], None, None]:
     ...
 
 
 @overload
-def grouped(
+def grouped[T, K](
     ms: Sequence[T],
-    key_func: Callable[[T], KT],
+    key_func: Callable[[T], K],
     keys_ordered: bool,
     values_only: True = True,
 ) -> Generator[list[T], None, None]:
@@ -381,20 +374,20 @@ def grouped(
 
 
 @overload
-def grouped(
+def grouped[T, K](
     ms: Sequence[T],
-    key_func: Callable[[T], KT],
+    key_func: Callable[[T], K],
     values_only: True = True,
 ) -> Generator[list[T], None, None]:
     ...
 
 
-def grouped(
+def grouped[T, K](
     ms: Sequence[T],
-    key_func: Callable[[T], KT],
+    key_func: Callable[[T], K],
     keys_ordered: bool = False,
     values_only: bool = False,
-) -> Generator[tuple[KT, list[T]] | list[T], None, None]:
+) -> Generator[tuple[K, list[T]] | list[T], None, None]:
     """
     Groups elements of a sequence by a key function.
 
@@ -406,7 +399,7 @@ def grouped(
     """
     if ms is None or len(ms) == 0:
         return
-    grouped_ms: dict[KT, list[T]] = {}
+    grouped_ms: dict[K, list[T]] = {}
     for m in ms:
         k = key_func(m)
         grouped_ms.setdefault(k, []).append(m)
@@ -414,7 +407,7 @@ def grouped(
     yield from (d for _, d in groups) if values_only else groups
 
 
-def deduped(ms: Sequence[T], comp_func: Callable[[T, T], bool]) -> Generator[T, None, None]:
+def deduped[T](ms: Sequence[T], comp_func: Callable[[T, T], bool]) -> Generator[T, None, None]:
     """
     Yields elements from the input sequence, removing consecutive duplicates as determined by the comparison function.
 
@@ -432,7 +425,7 @@ def deduped(ms: Sequence[T], comp_func: Callable[[T, T], bool]) -> Generator[T,
             prev = m
 
 
-def batched(ms: Iterable[T], batch_size: int) -> Generator[list[T], None, None]:
+def batched[T](ms: Iterable[T], batch_size: int) -> Generator[list[T], None, None]:
     """
     Splits an iterable into consecutive batches of a specified size.
 
@@ -456,7 +449,7 @@ def batched(ms: Iterable[T], batch_size: int) -> Generator[list[T], None, None]:
 batch_yield = batched
 
 
-def chunk(
+def chunk[T](
     ms: Sequence[T],
     chunk_func: Callable[[Sequence[T], T], bool],
     exclusive_end: bool = False,
@@ -485,7 +478,7 @@ def chunk(
     yield prev
 
 
-def chunk_between(
+def chunk_between[T](
     ms: Sequence[T],
     chunk_func: Callable[[T, T], bool],
     exclusive_end: bool = False,
@@ -504,9 +497,9 @@ def chunk_between(
     yield from chunk(ms, lambda x, y: chunk_func(last(x), y), exclusive_end)
 
 
-def chunk_with_key(
+def chunk_with_key[T, K](
     ms: Sequence[T],
-    key_func: Callable[[T], KT],
+    key_func: Callable[[T], K],
     exclusive_end: bool = False,
 ) -> Generator[list[T], None, None]:
     """
@@ -521,7 +514,7 @@ def chunk_with_key(
     yield from chunk_between(ms, lambda x, y: key_func(x) != key_func(y), exclusive_end)
 
 
-def merge_chunks(
+def merge_chunks[T](
     chunks: Sequence[Sequence[T]],
     merge_func: Callable[[Sequence[T], Sequence[T]], bool],
     drop_exclusive_end: bool = False,
@@ -546,11 +539,8 @@ def merge_chunks(
                    chunk_between(chunks, lambda a, b: not merge_func(a, b)))
 
 
-SeqT = TypeVar("SeqT", bound="Seq")
-
-
-class Seq(Generic[Tco], Sequence[Tco], Sized):
-    def __init__(self, data: Iterable[Tco] | SeqT):
+class Seq[T](Sequence[T], Sized):
+    def __init__(self, data: Iterable[T]):
         if isinstance(data, Seq):
             self.data = data.data
         elif isinstance(data, Iterable):
@@ -566,15 +556,15 @@ class Seq(Generic[Tco], Sequence[Tco], Sized):
     def empty(self) -> bool:
         return self.size == 0
 
-    def __add__(self: SeqT, other: SeqT) -> SeqT:
+    def __add__[SeqT: Seq](self: SeqT, other: SeqT) -> SeqT:
         return self.concat(other)
 
-    def __getitem__(self: SeqT, item: int | slice) -> SeqT:
+    def __getitem__[SeqT: Seq](self: SeqT, item: int | slice) -> SeqT:
         if isinstance(item, slice):
             return type(self)(self.data[item])
         elif isinstance(item, int):
             return type(self)(some(self.data[item]))
-        raise ValueError("unsupported index type")
+        raise IndexError("unsupported index type")
 
     def __len__(self) -> int:
         return len(self.data)
@@ -582,104 +572,104 @@ class Seq(Generic[Tco], Sequence[Tco], Sized):
     def __contains__(self, item) -> bool:
         return item in self.data
 
-    def __iter__(self) -> Iterator[Tco]:
+    def __iter__(self) -> Iterator[T]:
         return iter(self.data)
 
-    def __reversed__(self) -> Iterator[Tco]:
+    def __reversed__(self) -> Iterator[T]:
         return reversed(self.data)
 
-    def count(self, elem: Tco):
+    def count(self, elem: T):
         return self.count_if(lambda x: x == elem)
 
-    def count_if(self, func: Callable[[Tco], bool]):
+    def count_if(self, func: Callable[[T], bool]):
         return sum(1 for item in self.data if func(item))
 
-    def concat(self: SeqT, other: SeqT) -> SeqT:
+    def concat[SeqT: Seq](self: SeqT, other: SeqT) -> SeqT:
         return type(self)(itertools.chain(self.data, other.data))
 
-    def pad_left(self: SeqT, n: int, value: Tco) -> SeqT:
+    def pad_left[SeqT: Seq](self: SeqT, n: int, value: T) -> SeqT:
         return self if n <= 0 else type(self)(itertools.repeat(value, n)).concat(self)
 
-    def pad_right(self: SeqT, n: int, value: Tco) -> SeqT:
+    def pad_right[SeqT: Seq](self: SeqT, n: int, value: T) -> SeqT:
         return self if n <= 0 else self.concat(type(self)(itertools.repeat(value, n)))
 
-    def pad_left_head(self: SeqT, n: int) -> SeqT:
+    def pad_left_head[SeqT: Seq](self: SeqT, n: int) -> SeqT:
         return self if self.empty else self.pad_left(n, head(self.data))
 
-    def pad_right_last(self: SeqT, n: int) -> SeqT:
+    def pad_right_last[SeqT: Seq](self: SeqT, n: int) -> SeqT:
         return self if self.empty else self.pad_right(n, last(self.data))
 
-    def take_left(self: SeqT, n: int) -> SeqT:
+    def take_left[SeqT: Seq](self: SeqT, n: int) -> SeqT:
         if n <= 0:
             return type(self)([])
         return self[:n]
 
-    def take_right(self: SeqT, n: int) -> SeqT:
+    def take_right[SeqT: Seq](self: SeqT, n: int) -> SeqT:
         if n <= 0:
             return type(self)([])
         return self[-n:]
 
     take = take_left
 
-    def reverse(self: SeqT) -> SeqT:
+    def reverse[SeqT: Seq](self: SeqT) -> SeqT:
         return type(self)(reversed(self.data))
 
-    def distinct(self: SeqT) -> SeqT:
+    def distinct[SeqT: Seq](self: SeqT) -> SeqT:
         return type(self)(sorted(set(self.data)))
 
-    def scan_left(self, zero: TT | None, func: Callable[[TT, Tco], TT]) -> "Seq[TT]":
+    def scan_left[R](self, zero: R | None, func: Callable[[R, T], R]) -> "Seq[R]":
         return Seq(scan_left(self.data, zero, func))
 
-    def scan_right(self, zero: TT | None, func: Callable[[TT, Tco], TT]) -> "Seq[TT]":
+    def scan_right[R](self, zero: R | None, func: Callable[[R, T], R]) -> "Seq[R]":
         return Seq(scan_right(self.data, zero, func))
 
     scan = scan_left
 
-    def map(self, func: Callable[[Tco], TT]) -> "Seq[TT]":
+    def map[R](self, func: Callable[[T], R]) -> "Seq[R]":
         return self.scan(None, lambda x, y: func(y))
 
-    def fold_left(self, zero: TT | None, func: Callable[[TT, Tco], TT]) -> "Seq[TT]":
+    def fold_left[R](self, zero: R | None, func: Callable[[R, T], R]) -> "Seq[R]":
         return Seq(some(fold_left(self.data, zero, func)))
 
-    def fold_right(self, zero: TT | None, func: Callable[[TT, Tco], TT]) -> "Seq[TT]":
+    def fold_right[R](self, zero: R | None, func: Callable[[R, T], R]) -> "Seq[R]":
         return Seq(some(fold_right(self.data, zero, func)))
 
     fold = fold_left
 
-    def slide_left(
+    def slide_left[R](
         self,
         window: int,
-        func: Callable[[Iterable[Tco]], TT] = lambda x: x,
+        func: Callable[[Iterable[T]], R] = lambda x: x,
         *,
         allow_partial: tuple[bool, bool] = (False, False),
-        padding: tuple[Tco, Tco] = (None, None),
-    ) -> "Seq[TT]":
+        padding: tuple[T, T] = (None, None),
+    ) -> "Seq[R]":
         return Seq(slide_left(self.data, window, func, allow_partial=allow_partial, padding=padding))
 
-    def slide_right(
+    def slide_right[R](
         self,
         window: int,
-        func: Callable[[Iterable[Tco]], TT] = lambda x: x,
+        func: Callable[[Iterable[T]], R] = lambda x: x,
         *,
         allow_partial: tuple[bool, bool] = (False, False),
-        padding: tuple[Tco, Tco] = (None, None),
-    ) -> "Seq[TT]":
+        padding: tuple[T, T] = (None, None),
+    ) -> "Seq[R]":
         return Seq(slide_right(self.data, window, func, allow_partial=allow_partial, padding=padding))
 
     slide = slide_left
 
-    def reduce(self: SeqT, func: Callable[[Tco, Tco], Tco]) -> SeqT:
+    def reduce[SeqT: Seq](self: SeqT, func: Callable[[T, T], T]) -> SeqT:
         return type(self)(self.fold(None, lambda x, y: func(x, y)))
 
-    def max(self: SeqT, func: Callable[[Tco, Tco], bool] = None) -> SeqT:
+    def max[SeqT: Seq](self: SeqT, func: Callable[[T, T], bool] = None) -> SeqT:
         func = func or (lambda x, y: x > y)
         return self.reduce(lambda x, y: x if func(x, y) else y)
 
-    def min(self: SeqT, func: Callable[[Tco, Tco], bool] = None) -> SeqT:
+    def min[SeqT: Seq](self: SeqT, func: Callable[[T, T], bool] = None) -> SeqT:
         func = func or (lambda x, y: x < y)
         return self.reduce(lambda x, y: x if func(x, y) else y)
 
-    def group(self, func: Callable[[Tco], KT]) -> "Seq[tuple[KT, list[Tco]]]":
+    def group[K](self, func: Callable[[T], K]) -> "Seq[tuple[K, list[T]]]":
         return Seq(grouped(self.data, key_func=func, keys_ordered=True))
 
     def keys(self):
@@ -691,63 +681,63 @@ class Seq(Generic[Tco], Sequence[Tco], Sized):
     def swap(self):
         return Seq((value, key) for key, value in self.data)
 
-    def map_keys(self, func: Callable[[Tco], TT]):
+    def map_keys[R](self, func: Callable[[T], R]):
         return Seq((func(key), value) for key, value in self.data)
 
-    def map_values(self, func: Callable[[Tco], TT]):
+    def map_values[R](self, func: Callable[[T], R]):
         return Seq((key, func(value)) for key, value in self.data)
 
-    def flat_map(self, func: Callable[[Tco], Iterable[TT]]) -> "Seq[TT]":
+    def flat_map[R](self, func: Callable[[T], Iterable[R]]) -> "Seq[R]":
         return Seq(flatten(map(lambda x: func(x), self.data)))
 
     def flatten(self):
         return self.flat_map(lambda x: list(x))
 
-    def group_map(self, group_func: Callable[[Tco], KT], map_func: Callable[[Tco], TT]) -> "Seq[tuple[KT, list[TT]]]":
+    def group_map[K, R](self, group_func: Callable[[T], K], map_func: Callable[[T], R]) -> "Seq[tuple[K, list[R]]]":
         return self.group(group_func).map_values(lambda x: list(map(map_func, x)))
 
-    def filter(self: SeqT, func: Callable[[Tco], bool]) -> SeqT:
+    def filter[SeqT: Seq](self: SeqT, func: Callable[[T], bool]) -> SeqT:
         return type(self)(filter(func, self.data))
 
-    def filter_not(self: SeqT, func: Callable[[Tco], bool]) -> SeqT:
+    def filter_not[SeqT: Seq](self: SeqT, func: Callable[[T], bool]) -> SeqT:
         return self.filter(lambda x: not func(x))
 
-    def sort(self: SeqT, func: Callable[[Tco], KT]) -> SeqT:
+    def sort[SeqT: Seq, K](self: SeqT, func: Callable[[T], K]) -> SeqT:
         return type(self)(sorted(self.data, key=func))
 
-    def head(self: SeqT) -> SeqT:
+    def head[SeqT: Seq](self: SeqT) -> SeqT:
         return type(self)(some(head_or_none(self.data)))
 
-    def last(self: SeqT) -> SeqT:
+    def last[SeqT: Seq](self: SeqT) -> SeqT:
         return type(self)(some(last_or_none(self.data)))
 
-    def init(self: SeqT) -> SeqT:
+    def init[SeqT: Seq](self: SeqT) -> SeqT:
         return type(self)(init_iter(self.data))
 
-    def tail(self: SeqT) -> SeqT:
+    def tail[SeqT: Seq](self: SeqT) -> SeqT:
         return type(self)(tail_iter(self.data))
 
-    def foreach(self: SeqT, func: Callable[[Tco], None]) -> SeqT:
+    def foreach[SeqT: Seq](self: SeqT, func: Callable[[T], None]) -> SeqT:
         for elem in self.data:
             func(elem)
         return self
 
-    def exists(self, func: Callable[[Tco], bool]) -> "Seq[bool]":
+    def exists(self, func: Callable[[T], bool]) -> "Seq[bool]":
         return Seq(some(any(map(func, self.data))))
 
-    def forall(self, func: Callable[[Tco], bool]) -> "Seq[bool]":
+    def forall(self, func: Callable[[T], bool]) -> "Seq[bool]":
         return Seq(some(all(map(func, self.data))))
 
-    def union(self: SeqT, other: SeqT) -> SeqT:
+    def union[SeqT: Seq](self: SeqT, other: SeqT) -> SeqT:
         return type(self)(sorted(set(self.data).union(set(other.data))))
 
-    def intersect(self: SeqT, other: SeqT) -> SeqT:
+    def intersect[SeqT: Seq](self: SeqT, other: SeqT) -> SeqT:
         return type(self)(sorted(set(self.data).intersection(set(other.data))))
 
-    def zip(self, other: "Seq[TT]") -> "Seq[tuple[Tco, TT]]":
+    def zip[U](self, other: "Seq[U]") -> "Seq[tuple[T, U]]":
         return Seq(zip(self.data, other.data))
 
-    def zip_fill(self, other: "Seq[TT]", fill: Tco | TT | None = None) -> "Seq[tuple[Tco, TT]]":
+    def zip_fill[U](self, other: "Seq[U]", fill: T | U | None = None) -> "Seq[tuple[T, U]]":
         return Seq(itertools.zip_longest(self.data, other.data, fillvalue=fill))
 
 
