@@ -95,7 +95,11 @@ def path_depth(root: str, child: str) -> int:
     return child_expanded[len(root_expanded):].count(os.sep)
 
 
-def glob_match(names: list[str], include_patterns: list[str] = None, exclude_patterns: list[str] = None) -> list[str]:
+def glob_match(
+    names: list[str],
+    include_patterns: list[str] | None = None,
+    exclude_patterns: list[str] | None = None,
+) -> list[str]:
     """
     Applies the given inclusive and exclusive glob patterns to the given ``names`` and returns the filtered result.
 
@@ -121,8 +125,8 @@ class CopyFuncProtocol(Protocol):
 def listfile(
     path: str,
     *,
-    include_patterns: list[str] = None,
-    exclude_patterns: list[str] = None,
+    include_patterns: list[str] | None = None,
+    exclude_patterns: list[str] | None = None,
     depth: int = 0,
 ) -> list[str]:
     """
@@ -153,8 +157,8 @@ def copy(
     src: str,
     dst: str,
     *,
-    include_patterns: list[str] = None,
-    exclude_patterns: list[str] = None,
+    include_patterns: list[str] | None = None,
+    exclude_patterns: list[str] | None = None,
     depth: int = 0,
     follow_symlinks: bool = False,
     ignore_dangling_symlinks: bool = False,
