@@ -1,12 +1,14 @@
+import datetime
 import unittest
 
 import ddt
 
-from iker.common.utils.dtutils import basic_format, extended_format, iso_format, td_from_us, td_to_us
+from iker.common.utils.dtutils import basic_format, extended_format, iso_format
 from iker.common.utils.dtutils import dt_format, dt_format_iso, dt_parse, dt_parse_iso
 from iker.common.utils.dtutils import dt_from_ts, dt_from_ts_us, dt_to_ts, dt_to_ts_us
 from iker.common.utils.dtutils import dt_utc
 from iker.common.utils.dtutils import dt_utc_epoch, dt_utc_max, dt_utc_min
+from iker.common.utils.dtutils import td_from_time, td_from_us, td_to_time, td_to_us
 
 
 @ddt.ddt
@@ -44,6 +46,7 @@ class DtUtilsTest(unittest.TestCase):
         (1000000,),
         (60000000,),
         (3600000000,),
+        (86399999999,),
         (86400000000,),
         (31536000000000,),
     ]
@@ -53,6 +56,59 @@ class DtUtilsTest(unittest.TestCase):
     def test_td_us_roundtrip(self, data):
         self.assertEqual(data, td_to_us(td_from_us(data)))
 
+    data_td_us_roundtrip__td = [
+        (datetime.timedelta(),),
+        (datetime.timedelta(microseconds=1),),
+        (datetime.timedelta(microseconds=999),),
+        (datetime.timedelta(microseconds=1000),),
+        (datetime.timedelta(microseconds=999999),),
+        (datetime.timedelta(seconds=1),),
+        (datetime.timedelta(minutes=1),),
+        (datetime.timedelta(hours=1),),
+        (datetime.timedelta(hours=23, minutes=59, seconds=59, microseconds=999999),),
+        (datetime.timedelta(days=1),),
+        (datetime.timedelta(days=365),),
+    ]
+
+    @ddt.idata(data_td_us_roundtrip__td)
+    @ddt.unpack
+    def test_td_us_roundtrip__td(self, data):
+        self.assertEqual(data, td_from_us(td_to_us(data)))
+
+    data_td_time_roundtrip = [
+        (datetime.time(tzinfo=datetime.timezone.utc),),
+        (datetime.time(tzinfo=datetime.timezone.utc, microsecond=1),),
+        (datetime.time(tzinfo=datetime.timezone.utc, microsecond=999),),
+        (datetime.time(tzinfo=datetime.timezone.utc, microsecond=1000),),
+        (datetime.time(tzinfo=datetime.timezone.utc, microsecond=999999),),
+        (datetime.time(tzinfo=datetime.timezone.utc, second=1),),
+        (datetime.time(tzinfo=datetime.timezone.utc, minute=1),),
+        (datetime.time(tzinfo=datetime.timezone.utc, hour=1),),
+        (datetime.time(tzinfo=datetime.timezone.utc, hour=23, minute=59, second=59, microsecond=999999),),
+    ]
+
+    @ddt.idata(data_td_time_roundtrip)
+    @ddt.unpack
+    def test_td_time_roundtrip(self, data):
+        self.assertEqual(data, td_to_time(td_from_time(data)))
+
+    data_td_time_roundtrip__td = [
+        (datetime.timedelta(),),
+        (datetime.timedelta(microseconds=1),),
+        (datetime.timedelta(microseconds=999),),
+        (datetime.timedelta(microseconds=1000),),
+        (datetime.timedelta(microseconds=999999),),
+        (datetime.timedelta(seconds=1),),
+        (datetime.timedelta(minutes=1),),
+        (datetime.timedelta(hours=1),),
+        (datetime.timedelta(hours=23, minutes=59, seconds=59, microseconds=999999),),
+    ]
+
+    @ddt.idata(data_td_time_roundtrip__td)
+    @ddt.unpack
+    def test_td_time_roundtrip__td(self, data):
+        self.assertEqual(data, td_from_time(td_to_time(data)))
+
     data_dt_to_ts = [
         (dt_utc(1970, 1, 1), 0.0),
         (dt_utc(1969, 12, 31, 23, 59, 59), -1.0),
@@ -167,45 +223,45 @@ class DtUtilsTest(unittest.TestCase):
 
     data_dt_parse = [
         ("00010101T000000", basic_format(), dt_utc(1, 1, 1, 0, 0, 0)),
-        ("00010101T000000.000000", basic_format(with_ms=True), dt_utc(1, 1, 1, 0, 0, 0)),
+        ("00010101T000000.000000", basic_format(with_us=True), dt_utc(1, 1, 1, 0, 0, 0)),
         ("00010101T000000+0000", basic_format(with_tz=True), dt_utc(1, 1, 1, 0, 0, 0)),
-        ("00010101T000000.000000+0000", basic_format(with_ms=True, with_tz=True), dt_utc(1, 1, 1, 0, 0, 0)),
+        ("00010101T000000.000000+0000", basic_format(with_us=True, with_tz=True), dt_utc(1, 1, 1, 0, 0, 0)),
         ("19700101T000000", basic_format(), dt_utc(1970, 1, 1, 0, 0, 0)),
-        ("19700101T000000.000000", basic_format(with_ms=True), dt_utc(1970, 1, 1, 0, 0, 0)),
+        ("19700101T000000.000000", basic_format(with_us=True), dt_utc(1970, 1, 1, 0, 0, 0)),
         ("19700101T000000+0000", basic_format(with_tz=True), dt_utc(1970, 1, 1, 0, 0, 0)),
-        ("19700101T000000.000000+0000", basic_format(with_ms=True, with_tz=True), dt_utc(1970, 1, 1, 0, 0, 0)),
+        ("19700101T000000.000000+0000", basic_format(with_us=True, with_tz=True), dt_utc(1970, 1, 1, 0, 0, 0)),
         ("99991231T235959",
          basic_format(),
          dt_utc(9999, 12, 31, 23, 59, 59)),
         ("99991231T235959.000000",
-         basic_format(with_ms=True),
+         basic_format(with_us=True),
          dt_utc(9999, 12, 31, 23, 59, 59)),
         ("99991231T235959+0000",
          basic_format(with_tz=True),
          dt_utc(9999, 12, 31, 23, 59, 59)),
         ("99991231T235959.000000+0000",
-         basic_format(with_ms=True, with_tz=True),
+         basic_format(with_us=True, with_tz=True),
          dt_utc(9999, 12, 31, 23, 59, 59)),
 
         ("0001-01-01T00:00:00", extended_format(), dt_utc(1, 1, 1, 0, 0, 0)),
-        ("0001-01-01T00:00:00.000000", extended_format(with_ms=True), dt_utc(1, 1, 1, 0, 0, 0)),
+        ("0001-01-01T00:00:00.000000", extended_format(with_us=True), dt_utc(1, 1, 1, 0, 0, 0)),
         ("0001-01-01T00:00:00+00:00", extended_format(with_tz=True), dt_utc(1, 1, 1, 0, 0, 0)),
-        ("0001-01-01T00:00:00.000000+00:00", extended_format(with_ms=True, with_tz=True), dt_utc(1, 1, 1, 0, 0, 0)),
+        ("0001-01-01T00:00:00.000000+00:00", extended_format(with_us=True, with_tz=True), dt_utc(1, 1, 1, 0, 0, 0)),
         ("1970-01-01T00:00:00", extended_format(), dt_utc(1970, 1, 1, 0, 0, 0)),
-        ("1970-01-01T00:00:00.000000", extended_format(with_ms=True), dt_utc(1970, 1, 1, 0, 0, 0)),
+        ("1970-01-01T00:00:00.000000", extended_format(with_us=True), dt_utc(1970, 1, 1, 0, 0, 0)),
         ("1970-01-01T00:00:00+00:00", extended_format(with_tz=True), dt_utc(1970, 1, 1, 0, 0, 0)),
-        ("1970-01-01T00:00:00.000000+00:00", extended_format(with_ms=True, with_tz=True), dt_utc(1970, 1, 1, 0, 0, 0)),
+        ("1970-01-01T00:00:00.000000+00:00", extended_format(with_us=True, with_tz=True), dt_utc(1970, 1, 1, 0, 0, 0)),
         ("9999-12-31T23:59:59",
          extended_format(),
          dt_utc(9999, 12, 31, 23, 59, 59)),
         ("9999-12-31T23:59:59.000000",
-         extended_format(with_ms=True),
+         extended_format(with_us=True),
          dt_utc(9999, 12, 31, 23, 59, 59)),
         ("9999-12-31T23:59:59+00:00",
          extended_format(with_tz=True),
          dt_utc(9999, 12, 31, 23, 59, 59)),
         ("9999-12-31T23:59:59.000000+00:00",
-         extended_format(with_ms=True, with_tz=True),
+         extended_format(with_us=True, with_tz=True),
          dt_utc(9999, 12, 31, 23, 59, 59)),
 
         ("0001-01-01T00:00:00", "%Y-%m-%dT%H:%M:%S", dt_utc(1, 1, 1, 0, 0, 0)),
@@ -248,45 +304,45 @@ class DtUtilsTest(unittest.TestCase):
 
     data_dt_format = [
         (dt_utc(1, 1, 1, 0, 0, 0), basic_format(), "00010101T000000"),
-        (dt_utc(1, 1, 1, 0, 0, 0), basic_format(with_ms=True), "00010101T000000.000000"),
+        (dt_utc(1, 1, 1, 0, 0, 0), basic_format(with_us=True), "00010101T000000.000000"),
         (dt_utc(1, 1, 1, 0, 0, 0), basic_format(with_tz=True), "00010101T000000+0000"),
-        (dt_utc(1, 1, 1, 0, 0, 0), basic_format(with_ms=True, with_tz=True), "00010101T000000.000000+0000"),
+        (dt_utc(1, 1, 1, 0, 0, 0), basic_format(with_us=True, with_tz=True), "00010101T000000.000000+0000"),
         (dt_utc(1970, 1, 1, 0, 0, 0), basic_format(), "19700101T000000"),
-        (dt_utc(1970, 1, 1, 0, 0, 0), basic_format(with_ms=True), "19700101T000000.000000"),
+        (dt_utc(1970, 1, 1, 0, 0, 0), basic_format(with_us=True), "19700101T000000.000000"),
         (dt_utc(1970, 1, 1, 0, 0, 0), basic_format(with_tz=True), "19700101T000000+0000"),
-        (dt_utc(1970, 1, 1, 0, 0, 0), basic_format(with_ms=True, with_tz=True), "19700101T000000.000000+0000"),
+        (dt_utc(1970, 1, 1, 0, 0, 0), basic_format(with_us=True, with_tz=True), "19700101T000000.000000+0000"),
         (dt_utc(9999, 12, 31, 23, 59, 59),
          basic_format(),
          "99991231T235959"),
         (dt_utc(9999, 12, 31, 23, 59, 59),
-         basic_format(with_ms=True),
+         basic_format(with_us=True),
          "99991231T235959.000000"),
         (dt_utc(9999, 12, 31, 23, 59, 59),
          basic_format(with_tz=True),
          "99991231T235959+0000"),
         (dt_utc(9999, 12, 31, 23, 59, 59),
-         basic_format(with_ms=True, with_tz=True),
+         basic_format(with_us=True, with_tz=True),
          "99991231T235959.000000+0000"),
 
         (dt_utc(1, 1, 1, 0, 0, 0), extended_format(), "0001-01-01T00:00:00"),
-        (dt_utc(1, 1, 1, 0, 0, 0), extended_format(with_ms=True), "0001-01-01T00:00:00.000000"),
+        (dt_utc(1, 1, 1, 0, 0, 0), extended_format(with_us=True), "0001-01-01T00:00:00.000000"),
         (dt_utc(1, 1, 1, 0, 0, 0), extended_format(with_tz=True), "0001-01-01T00:00:00+00:00"),
-        (dt_utc(1, 1, 1, 0, 0, 0), extended_format(with_ms=True, with_tz=True), "0001-01-01T00:00:00.000000+00:00"),
+        (dt_utc(1, 1, 1, 0, 0, 0), extended_format(with_us=True, with_tz=True), "0001-01-01T00:00:00.000000+00:00"),
         (dt_utc(1970, 1, 1, 0, 0, 0), extended_format(), "1970-01-01T00:00:00"),
-        (dt_utc(1970, 1, 1, 0, 0, 0), extended_format(with_ms=True), "1970-01-01T00:00:00.000000"),
+        (dt_utc(1970, 1, 1, 0, 0, 0), extended_format(with_us=True), "1970-01-01T00:00:00.000000"),
         (dt_utc(1970, 1, 1, 0, 0, 0), extended_format(with_tz=True), "1970-01-01T00:00:00+00:00"),
-        (dt_utc(1970, 1, 1, 0, 0, 0), extended_format(with_ms=True, with_tz=True), "1970-01-01T00:00:00.000000+00:00"),
+        (dt_utc(1970, 1, 1, 0, 0, 0), extended_format(with_us=True, with_tz=True), "1970-01-01T00:00:00.000000+00:00"),
         (dt_utc(9999, 12, 31, 23, 59, 59),
          extended_format(),
          "9999-12-31T23:59:59"),
         (dt_utc(9999, 12, 31, 23, 59, 59),
-         extended_format(with_ms=True),
+         extended_format(with_us=True),
          "9999-12-31T23:59:59.000000"),
         (dt_utc(9999, 12, 31, 23, 59, 59),
          extended_format(with_tz=True),
          "9999-12-31T23:59:59+00:00"),
         (dt_utc(9999, 12, 31, 23, 59, 59),
-         extended_format(with_ms=True, with_tz=True),
+         extended_format(with_us=True, with_tz=True),
          "9999-12-31T23:59:59.000000+00:00"),
 
         (dt_utc(9999, 12, 31, 23, 59, 59), "%Y-%m-%dT%H:%M:%S", "9999-12-31T23:59:59"),
