@@ -4,6 +4,7 @@ import unittest
 import ddt
 
 from iker.common.utils.funcutils import composable, identity
+from iker.common.utils.funcutils import const, first, packed, second
 from iker.common.utils.funcutils import lazy, memorized, singleton, unique_returns
 from iker.common.utils.randutils import randomizer
 
@@ -19,6 +20,45 @@ class Counter(object):
 @ddt.ddt
 class FuncUtilsTest(unittest.TestCase):
 
+    def test_const(self):
+        self.assertEqual(5, const(5)())
+        self.assertEqual(5, const(5)(1, 2, 3))
+        self.assertEqual(5, const(5)(a=1, b=2))
+
+        self.assertEqual("hello", const("hello")())
+        self.assertEqual("hello", const("hello")(1, 2, 3))
+        self.assertEqual("hello", const("hello")(a=1, b=2))
+
+    def test_first(self):
+        self.assertEqual(1, first()((1, "a")))
+        self.assertEqual("key", first()(("key", 12)))
+        self.assertEqual((1, 2), first()(((1, 2), "value")))
+
+        with self.assertRaises(ValueError):
+            first()((1, 2, 3))
+        with self.assertRaises(ValueError):
+            first()(123)
+
+    def test_second(self):
+        self.assertEqual("a", second()((1, "a")))
+        self.assertEqual(12, second()(("key", 12)))
+        self.assertEqual("value", second()(((1, 2), "value")))
+
+        with self.assertRaises(ValueError):
+            second()((1, 2, 3))
+        with self.assertRaises(ValueError):
+            second()(123)
+
+    def test_packed(self):
+        self.assertEqual(3, packed(lambda x, y: x + y)((1, 2)))
+        self.assertEqual(12, packed(lambda x, y, z: x + y + z)((3, 4, 5)))
+        self.assertEqual("aab", packed(lambda a, b, x, y: a * x + b * y)(("a", "b", 2, 1)))
+
+        with self.assertRaises(TypeError):
+            packed(lambda x, y, z: x ** y ** z)((2, 3, 4, 5))
+        with self.assertRaises(TypeError):
+            packed(lambda x, y, z: x ** y ** z)((2, 3))
+
     def test_identity(self):
         self.assertEqual(1, identity(1))
         self.assertEqual("test", identity("test"))
