@@ -8,7 +8,7 @@ from typing import SupportsFloat, SupportsInt
 import ddt
 
 from iker.common.utils.jsonutils import asterisk, json_get, json_set
-from iker.common.utils.jsonutils import json_compare
+from iker.common.utils.jsonutils import json_compare, json_difference, json_equals
 from iker.common.utils.jsonutils import json_reformat, json_sanitize, json_traverse
 from iker.common.utils.randutils import randomizer
 
@@ -1189,3 +1189,286 @@ class JsonUtilsTest(unittest.TestCase):
     @ddt.unpack
     def test_json_sanitize(self, data, expect):
         self.assertTrue(json_compare(json_sanitize(data), expect))
+
+    data_json_difference = [
+        (
+            {
+                "none": None,
+                "bool": {
+                    "bool_true": True,
+                    "bool_false": False,
+                },
+                "int": {
+                    "int_one": 1,
+                    "int_minus_one": -1,
+                    "int_zero": 0,
+                    "int_wrapped": MultipliedInt(1, -1000),
+                },
+                "float": {
+                    "float_one": 1.0,
+                    "float_minus_one": -1.0,
+                    "float_zero": 0.0,
+                    "float_one_e_nine": 1.0e9,
+                    "float_minus_one_e_minus_nine": -1.0e-9,
+                    "float_nan": math.nan,
+                    "float_inf": math.inf,
+                    "float_minus_inf": -math.inf,
+                    "float_wrapped": MultipliedFloat(1.0, -1000.0),
+                },
+                "str": {
+                    "str_empty": "",
+                    "str": "dummy",
+                },
+                "list":
+                    [None, True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"],
+                "tuple":
+                    (None, True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"),
+                "none_not_none": None,
+                "bool_negated": {
+                    "bool_true": True,
+                    "bool_false": False,
+                },
+                "int_negated": {
+                    "int_one": 1,
+                    "int_minus_one": -1,
+                    "int_wrapped": MultipliedInt(1, -1000),
+                },
+                "float_negated": {
+                    "float_one": 1.0,
+                    "float_minus_one": -1.0,
+                    "float_one_e_nine": 1.0e9,
+                    "float_minus_one_e_minus_nine": -1.0e-9,
+                    "float_inf": math.inf,
+                    "float_minus_inf": -math.inf,
+                    "float_wrapped": MultipliedFloat(1.0, -1000.0),
+                },
+                "num_type_mismatched": {
+                    "int_as_float": 1,
+                    "float_as_int": 1.0,
+                },
+                "str_unidentical": {
+                    "str_empty": "",
+                    "str": "dummy",
+                },
+                "list_size_unidentical":
+                    [None, True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"],
+                "list_order_changed":
+                    [None, True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"],
+                "list_first_item_mismatched":
+                    [None, True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"],
+                "tuple_size_unidentical":
+                    (None, True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"),
+                "tuple_order_changed":
+                    (None, True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"),
+                "tuple_first_item_mismatched":
+                    (None, True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"),
+                "dict_size_unidentical": {
+                    "none": None,
+                    "bool_true": True,
+                    "bool_false": False,
+                    "int_one": 1,
+                    "int_minus_one": -1,
+                    "int_zero": 0,
+                    "float_one": 1.0,
+                    "float_minus_one": -1.0,
+                    "float_zero": 0.0,
+                    "float_one_e_nine": 1.0e9,
+                    "float_minus_one_e_minus_nine": -1.0e-9,
+                    "float_nan": "nan",
+                    "float_inf": "inf",
+                    "float_minus_inf": "-inf",
+                    "str_empty": "",
+                    "str": "dummy",
+                },
+                "dict_keys_unidentical": {
+                    "none": None,
+                    "bool_true": True,
+                    "bool_false": False,
+                    "int_one": 1,
+                    "int_minus_one": -1,
+                    "int_zero": 0,
+                    "float_one": 1.0,
+                    "float_minus_one": -1.0,
+                    "float_zero": 0.0,
+                    "float_one_e_nine": 1.0e9,
+                    "float_minus_one_e_minus_nine": -1.0e-9,
+                    "float_nan": "nan",
+                    "float_inf": "inf",
+                    "float_minus_inf": "-inf",
+                    "str_empty": "",
+                    "str": "dummy",
+                },
+            },
+            {
+                "none": None,
+                "bool": {
+                    "bool_true": True,
+                    "bool_false": False,
+                },
+                "int": {
+                    "int_one": 1,
+                    "int_minus_one": -1,
+                    "int_zero": 0,
+                    "int_wrapped": -1000,
+                },
+                "float": {
+                    "float_one": 1.0,
+                    "float_minus_one": -1.0,
+                    "float_zero": 0.0,
+                    "float_one_e_nine": 1.0e9,
+                    "float_minus_one_e_minus_nine": -1.0e-9,
+                    "float_nan": math.nan,
+                    "float_inf": math.inf,
+                    "float_minus_inf": -math.inf,
+                    "float_wrapped": -1000.0,
+                },
+                "str": {
+                    "str_empty": "",
+                    "str": "dummy",
+                },
+                "list":
+                    [None, True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"],
+                "tuple":
+                    [None, True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"],
+                "none_not_none": "",  # ["none_not_none"]
+                "bool_negated": {
+                    "bool_true": False,  # ["bool_negated", "bool_true"]
+                    "bool_false": True,  # ["bool_negated", "bool_false"]
+                },
+                "int_negated": {
+                    "int_one": -1,  # ["int_negated", "int_one"]
+                    "int_minus_one": 1,  # ["int_negated", "int_minus_one"]
+                    "int_wrapped": 1000,  # ["int_negated", "int_wrapped"]
+                },
+                "float_negated": {
+                    "float_one": -1.0,  # ["float_negated", "float_one"]
+                    "float_minus_one": 1.0,  # ["float_negated", "float_minus_one"]
+                    "float_one_e_nine": -1.0e9,  # ["float_negated", "float_one_e_nine"]
+                    "float_minus_one_e_minus_nine": 1.0e-9,  # Passes the equality check w.r.t. float tolerance
+                    "float_inf": -math.inf,  # ["float_negated", "float_inf"]
+                    "float_minus_inf": math.inf,  # ["float_negated", "float_minus_inf"]
+                    "float_wrapped": 1000.0,  # ["float_negated", "float_wrapped"]
+                },
+                "num_type_mismatched": {
+                    "int_as_float": 1.0,  # ["num_type_mismatched", "int_as_float"]
+                    "float_as_int": 1,  # ["num_type_mismatched", "float_as_int"]
+                },
+                "str_unidentical": {
+                    "str_empty": "dummy",  # ["str_unidentical", "str_empty"]
+                    "str": "",  # ["str_unidentical", "str"]
+                },
+                "list_size_unidentical":  # ["list_size_unidentical"]
+                    [True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"],
+                "list_order_changed":  # ["list_order_changed", 0...15], 16 mismatches caused by left shift by one
+                    [True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy", None],
+                "list_first_item_mismatched":  # ["list_first_item_mismatched", 0]
+                    ["", True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"],
+                "tuple_size_unidentical":  # ["tuple_size_unidentical"]
+                    (True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"),
+                "tuple_order_changed":  # ["tuple_order_changed", 0...15], 16 mismatches caused by left shift by one
+                    (True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy", None),
+                "tuple_first_item_mismatched":  # ["tuple_first_item_mismatched", 0]
+                    ("", True, False, 1, -1, 0, 1., -1., 0., 1e9, -1e-9, math.nan, math.inf, -math.inf, "", "dummy"),
+                "dict_size_unidentical":  # ["dict_size_unidentical"]
+                    {
+                        "bool_true": True,
+                        "bool_false": False,
+                        "int_one": 1,
+                        "int_minus_one": -1,
+                        "int_zero": 0,
+                        "float_one": 1.0,
+                        "float_minus_one": -1.0,
+                        "float_zero": 0.0,
+                        "float_one_e_nine": 1.0e9,
+                        "float_minus_one_e_minus_nine": -1.0e-9,
+                        "float_nan": "nan",
+                        "float_inf": "inf",
+                        "float_minus_inf": "-inf",
+                        "str_empty": "",
+                        "str": "dummy",
+                    },
+                "dict_keys_unidentical":  # ["dict_keys_unidentical"]
+                    {
+                        "another_none": None,
+                        "another_bool_true": True,
+                        "another_bool_false": False,
+                        "another_int_one": 1,
+                        "another_int_minus_one": -1,
+                        "another_int_zero": 0,
+                        "another_float_one": 1.0,
+                        "another_float_minus_one": -1.0,
+                        "another_float_zero": 0.0,
+                        "another_float_one_e_nine": 1.0e9,
+                        "another_float_minus_one_e_minus_nine": -1.0e-9,
+                        "another_float_nan": "nan",
+                        "another_float_inf": "inf",
+                        "another_float_minus_inf": "-inf",
+                        "another_str_empty": "",
+                        "another_str": "dummy",
+                    },
+            },
+            [
+                ["none_not_none"],
+                ["bool_negated", "bool_true"],
+                ["bool_negated", "bool_false"],
+                ["int_negated", "int_one"],
+                ["int_negated", "int_minus_one"],
+                ["int_negated", "int_wrapped"],
+                ["float_negated", "float_one"],
+                ["float_negated", "float_minus_one"],
+                ["float_negated", "float_one_e_nine"],
+                ["float_negated", "float_inf"],
+                ["float_negated", "float_minus_inf"],
+                ["float_negated", "float_wrapped"],
+                ["num_type_mismatched", "int_as_float"],
+                ["num_type_mismatched", "float_as_int"],
+                ["str_unidentical", "str_empty"],
+                ["str_unidentical", "str"],
+                ["list_size_unidentical"],
+                ["list_order_changed", 0],
+                ["list_order_changed", 1],
+                ["list_order_changed", 2],
+                ["list_order_changed", 3],
+                ["list_order_changed", 4],
+                ["list_order_changed", 5],
+                ["list_order_changed", 6],
+                ["list_order_changed", 7],
+                ["list_order_changed", 8],
+                ["list_order_changed", 9],
+                ["list_order_changed", 10],
+                ["list_order_changed", 11],
+                ["list_order_changed", 12],
+                ["list_order_changed", 13],
+                ["list_order_changed", 14],
+                ["list_order_changed", 15],
+                ["list_first_item_mismatched", 0],
+                ["tuple_size_unidentical"],
+                ["tuple_order_changed", 0],
+                ["tuple_order_changed", 1],
+                ["tuple_order_changed", 2],
+                ["tuple_order_changed", 3],
+                ["tuple_order_changed", 4],
+                ["tuple_order_changed", 5],
+                ["tuple_order_changed", 6],
+                ["tuple_order_changed", 7],
+                ["tuple_order_changed", 8],
+                ["tuple_order_changed", 9],
+                ["tuple_order_changed", 10],
+                ["tuple_order_changed", 11],
+                ["tuple_order_changed", 12],
+                ["tuple_order_changed", 13],
+                ["tuple_order_changed", 14],
+                ["tuple_order_changed", 15],
+                ["tuple_first_item_mismatched", 0],
+                ["dict_size_unidentical"],
+                ["dict_keys_unidentical"],
+            ]
+        )
+    ]
+
+    @ddt.idata(data_json_difference)
+    @ddt.unpack
+    def test_json_difference(self, a, b, expect_diffs):
+        diffs = list(node_path for node_path, _ in json_difference(a, b, int_strict=True))
+        for diff, expect_diff in zip(sorted(diffs), sorted(expect_diffs)):
+            self.assertTrue(json_equals(diff, expect_diff))
